package com.biganto.visual.roompark.base

import android.os.Bundle
import android.view.View
import android.view.ViewGroup
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.widget.Toolbar
import androidx.coordinatorlayout.widget.CoordinatorLayout
import butterknife.BindView
import com.biganto.visual.roompark.BuildConfig
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.view_utils.app_bar.DragControlAppBarLayoutBehaviour
import com.biganto.visual.roompark.view_utils.snackbar.ISnackBarProvider
import com.bluelinelabs.conductor.Conductor
import com.bluelinelabs.conductor.Router
import com.crashlytics.android.Crashlytics
import com.google.android.material.appbar.AppBarLayout
import com.google.android.material.appbar.CollapsingToolbarLayout
import com.google.android.material.bottomnavigation.BottomNavigationView
import io.fabric.sdk.android.Fabric
import org.koin.android.scope.currentScope

class RoomParkMainActivity(
) : AppCompatActivity()
    ,ICollapsingToolBar
    ,IConductorActivity
    ,IBottomNavigation{

    private val snackbarProvider: ISnackBarProvider by currentScope.inject()

    private lateinit var router: Router

    @BindView(R.id.top_toolbar) override lateinit var topAppBar: Toolbar
    @BindView(R.id.app_bar) override lateinit var appBar: AppBarLayout
    @BindView(R.id.topToolbarHolder) override lateinit var coordinatorLayout: CoordinatorLayout
    @BindView(R.id.collapsingToolbarLayout) override lateinit var collapsingToolbarLayout: CollapsingToolbarLayout
    @BindView(R.id.conductor_container) override lateinit var conductorContainer: ViewGroup
    @BindView(R.id.bottom_navigation_view) override lateinit var bottomNavigation: BottomNavigationView

    override fun onCreate(savedInstanceState: Bundle?) {
        setTheme(R.style.AppTheme)
        super.onCreate(savedInstanceState)

        if (!BuildConfig.DEBUG) Fabric.with(this, Crashlytics())

        setContentView(R.layout.activity_main)

        setSupportActionBar(topAppBar)

        router = Conductor.attachRouter(this, conductorContainer, savedInstanceState)

        snackbarProvider.bindRootView(rootView = coordinatorLayout)
    }

    override fun displayBackButton(show: Boolean) {
        supportActionBar?.setDisplayHomeAsUpEnabled(show)
    }

    override fun hideAll() {
        appBar.visibility=Toolbar.INVISIBLE
        bottomNavigation.visibility = Toolbar.GONE
        val params = coordinatorLayout.layoutParams as CoordinatorLayout.LayoutParams
        params.behavior = null
        coordinatorLayout.requestLayout()
    }

    override fun showAll() {
        appBar.setExpanded(false,false)
        collapsingToolbarLayout.visibility= View.VISIBLE
        bottomNavigation.visibility = Toolbar.VISIBLE
        appBar.visibility= View.VISIBLE
        topAppBar.visibility = View.VISIBLE

        val params = coordinatorLayout.layoutParams as CoordinatorLayout.LayoutParams
        params.behavior = AppBarLayout.ScrollingViewBehavior()
        coordinatorLayout.requestLayout()
    }

    override fun appBarScrollable(allow: Boolean) {
        val params = appBar.layoutParams as CoordinatorLayout.LayoutParams
        val behavior = params.behavior as DragControlAppBarLayoutBehaviour
        behavior.allowDrag=allow
    }
}
