package com.biganto.visual.roompark.conductor

import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.InputMethodManager
import android.widget.Toast
import androidx.annotation.LayoutRes
import androidx.annotation.StringRes
import androidx.appcompat.app.ActionBar
import androidx.appcompat.app.AppCompatActivity
import butterknife.ButterKnife
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.conductor.mosby.mvi.BigantoMviController
import com.biganto.visual.roompark.view_utils.snackbar.ISnackBarProvider
import com.hannesdorfmann.mosby3.mvi.MviBasePresenter
import io.reactivex.disposables.CompositeDisposable

/**
 * Created by Vladislav Bogdashkin on 28.05.2018.
 */
abstract class BigantoBaseController<VS : BigantoBaseViewState,V: BigantoBaseContract<VS>,P : MviBasePresenter<V,VS>>
    : BigantoMviController<V, P> {

    constructor():super()

    constructor(args: Bundle):super(args)

    protected val detachDisposable = CompositeDisposable()

    abstract override fun createPresenter(): P

    @LayoutRes
    protected abstract fun getLayoutId(): Int

    @StringRes
    protected abstract fun getToolbarTitleId(): Int

    protected abstract fun showBottomAppBar(): Boolean

//    protected lateinit var toolBars:TopBottomActionBarsSupported

    protected lateinit var snackbar: ISnackBarProvider

    private fun setToolbarTitle() {
        // if the Activity happens to be non-AppCompatActivity or it does not have ActionBar, simply do not set the title
        (activity as? AppCompatActivity)?.supportActionBar?.apply {
            title = resources?.getString(getToolbarTitleId())
            setDisplayHomeAsUpEnabled(router.backstackSize > 1)
        }
    }

    //TODO(injection)
    private fun bottomToolbar() : ActionBar?=(activity as? AppCompatActivity)?.supportActionBar

    override fun onDetach(view: View) {
        detachDisposable.clear()
        super.onDetach(view)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup): View {
        val view = inflater.inflate(getLayoutId(), container, false)
//        instantiate the view
        ButterKnife.bind(this, view)
//        toolBars=(activity as TopBottomActionBarsSupported)

        snackbar = activity as ISnackBarProvider
        //TODO(remove duplicated code:)
        if (showBottomAppBar())
            bottomToolbar()?.show()
        if (!showBottomAppBar())
            bottomToolbar()?.hide()
        return view
    }

    protected fun View.hideKeyboard() {
        val inputMethodManager = applicationContext?.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        inputMethodManager.hideSoftInputFromWindow(this.windowToken, 0)
    }

    protected fun View.showKeyboard() {
        val inputMethodManager = applicationContext?.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        inputMethodManager.showSoftInput(this, 0)
    }

    protected fun showMessage(@StringRes msgResId: Int) {
        Toast.makeText(this.applicationContext, msgResId, Toast.LENGTH_SHORT).show()
    }

    override fun handleBack(): Boolean {
        return super.handleBack()
    }

    protected val isTablet
        get() = resources?.getBoolean(R.bool.isTablet)?:false

    protected val catalogSpansCount
        get() = resources?.getInteger(R.integer.catalogSpans)?:1

    protected val toursSpansCount
        get() = resources?.getInteger(R.integer.catalogSpans)?:1
}